C-----------------------------------------------------------------------
      module highlevel
C   Lithium version for solid
C   Here: quantities as parameters, variables on highest program level
       implicit none
C   Double Precision
       integer, parameter,public ::
     & dp=selected_real_kind(2*precision(1.0))
       real(dp),parameter,public :: EMACH=1.0e-6_dp,
     &                      PI=3.1415926535897932_dp
C   Physical constants
       real(dp),parameter,public :: HARTREE=27.21168_dp,
     &   BOHR=0.52917706_dp
C  Lithium has a bcc lattice with experimental cubic lattice constant
C    of 3.4769(3) Angstroem.
C  LATTY type of nuclei lattice
C  LATCON lattice constant of cubic physical lattice in a.u.
C  SULATCON lattice constant of superlattice in a.u.
C  SULATTY type of superlattice for simulation with strict periodicity,
C    i.e. also the wavefunction is periodic with unit cells of SULATTY;
C    for Li solid LATTY=3,SULATTY=1
C  SC,FCC,BCC lattice types numbered by 1,2,3
C  MLAT number of lattice types for dimensioning
C  N1D number of SC unit cells of LATTY along one cube axis
C    contained in the supercell SULATTY; N1D**3 is their total number
C  NZ number of SC cells in the supercell which is equal to N1D**3
C  MZ number of electrons per nucleus
C  NE total number electrons=2*MZ*N1D**3=MZ*NK
C  NES(2) number electrons per spin, 1=up-spin, 2=down-spin,
C    use NES(1) >= NES(2) and put NES1=NES(1) for dimensioning
C  NK number of nuclei in the supercell which is equal to 2*N1D**3;
C    note that only one half of the supercell surface counts to the cell
C  CHARGE nuclear charge (positive)
       integer,parameter,public :: SC=1,FCC=2,BCC=3,MLAT=3
       integer,parameter,public :: LATTY=3,SULATTY=1
       integer,parameter,public :: N1D=2,MZ=3,NZ=N1D*N1D*N1D
       integer,parameter,public :: NEMAX=48,NKMAX=16,NE=2*NZ*MZ,NK=16
       integer,parameter,dimension(2),public  :: NES=(/24,24/)
       integer,parameter,public :: NES1=24
       integer,public :: MCPRE,MCMAX,MAXA
       real(dp),parameter,public :: LATCON=6.5704_dp,CHARGE=3._dp
       real(dp),public           :: SULATCON=LATCON*N1D
      end module highlevel
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module midlevel
       use highlevel
C  Here: quantities may change during run
       implicit none
       SAVE
C  IE=1,NE index of electrons
C  IES=1,2 index of spin: 1=up, 2=down
C  IEES=1,NES(IES) index of electrons of same spin
C  IENS=NES(IES) no. of electrons with specified spin
C  IK=1,NK index of nuclei
C  SPINSEL= spin select
       integer,public :: IE,IES,IK,IEES,IENS,IMC,IMCR,IMCA,IBLOCKA,
     &                   MCOUNT,SPINSEL,IMCOPT
       logical,public :: MCSTEP,MCRUN
C  RE position array of electron
C  RNEW position of electron after move
C  RK position array of nuclei
C  DIST actual distances between electrons,4th component for modulus
C  DISTNEW updated distances from a moved electron
C  JASEMACH lowest distance used in Jastrow factor for finiteness
       real(dp),parameter,public          :: JASEMACH=2.D0/3.D0*EMACH
       real(dp),parameter,public          :: KINEMACH=3.D0/EMACH
       integer,dimension(NE),public       :: IKM,IKMOLD
       real(dp),public                    :: STEPMAX
       real(dp),public       :: MINOPT,EGUESS,OPTVAR,WPSI,SWEIGHT
       real(dp),dimension(3,NE),public    :: RE,RENEW,ZE,ZENEW
       real(dp),dimension(3,NK),public    :: RK,ZK
       real(dp),dimension(3,NK,NE),public :: ZKE,ZKENEW
       real(dp),dimension(3,NE,NE),public :: ZEE
       real(dp),dimension(4,NK,NE),public :: RKE,RKENEW
       real(dp),dimension(4,NE,NE),public :: DIST,DISTNEW,REE
       real(kind=dp),public,dimension(NES1,NES1,2)  :: PSIMAT,PSIMATOPT
       public :: RDIST,RDISTELNU,RDISTELEL,FOLDTOWSC
      contains
C-----------------------------------------------------------------------
      subroutine RDIST(r,rn)
C  Difference vector and distance between two arrays
C  each of them numbered by one electron index. Stores in two global
C  arrays DIST(1:4,1:NE,1:NE) and DISTNEW(1:4,1:NE,1:NE)
C  with one index fixed to the actual electron IE.
       real(kind=dp),dimension(3,NE),intent(in) :: r,rn
C
       integer :: i,k,n
       real(dp) :: woo,won
       ielek:do k=1,NE
         if (k .eq. IE) then
          cycle ielek
         end if
          DIST(1:3,IE,k)=r(1:3,IE)-r(1:3,k)
          DISTNEW(1:3,IE,k)=rn(1:3,IE)-r(1:3,k)
          woo=dsqrt (sum (DIST(1:3,IE,k)**2))
          won=dsqrt (sum (DISTNEW(1:3,IE,k)**2))
C  Cut-off at small distance for Jastrow factor
         if (woo .lt. EMACH)  woo = JASEMACH
         if (won .lt. EMACH)  won = JASEMACH
         DIST(4,IE,k) = woo
         DISTNEW(4,IE,k) = won
       end do ielek
      end subroutine RDIST
C-----------------------------------------------------------------------
      subroutine RDISTELNU(el,nu,ren)
C  Calculates the difference vectors between arrays e and k to give r,
C  folds them back into Wigner-Seitz cell of the superlattice
C  and outputs the distance as 4th component of r.
       real(dp),intent(in),dimension(3)         :: el
       real(dp),intent(in),dimension(3,NK)      :: nu
       real(dp),intent(out),dimension(4,NK)     :: ren
C
       integer                                  :: ij,ik
       real(dp),dimension(3)                    :: z
C
       do ik=1,NK
        ren(1:3,ik) = el(1:3) - nu(1:3,ik)
        z(1:3) = ren(1:3,ik)/SULATCON
        ren(4,ik) = sqrt (sum (ren(1:3,ik)**2))
       end do
      end subroutine RDISTELNU
C-----------------------------------------------------------------------
      subroutine RDISTELEL(ei,ek,ree,zee)
C  Calculates the difference vectors between arrays e and k to give r,
C  folds them back into Wigner-Seitz cell of the superlattice
C  and outputs the distance as 4th component of r. Coordinates z in
C  units of superlattice cell are given, too.
       real(dp),intent(in),dimension(3,NE)      :: ei,ek
       real(dp),intent(out),dimension(3,NE,NE)  :: zee
       real(dp),intent(out),dimension(4,NE,NE)  :: ree
C
       integer  :: ij,ik
       do ij=1,NE
        do ik=1,NE
         ree(1:3,ik,ij) = ei(1:3,ij) - ek(1:3,ik)
         zee(1:3,ik,ij) = ree(1:3,ik,ij)/SULATCON
         ree(4,ik,ij) = sqrt (sum (ree(1:3,ik,ij)**2))
        end do
       end do
      end subroutine RDISTELEL
C-----------------------------------------------------------------------
      subroutine FOLDTOWSC(zpos)
C  Folds argument zpos back into the Wigner-Seitz cell of a sc-lattice
C  with lattice constant 1.
       real(dp),intent(inout),dimension(3)  :: zpos
C
       integer  ::  i
       real(dp) ::  a
       do,i=1,3
        a=zpos(i) - anint (zpos(i))
C  Observe: anint rounds to the nearest integer, but n.5 goes
C  to (n+1).0 for n>0 and (n-1).0 for n<0. Only surfaces on positive
C  sides belong to Wigner-Seitz cell
        if (abs (a) == 0.5_dp) then
         zpos(i) = zpos(i) - EMACH
         zpos(i) = zpos(i) - anint (zpos(i))
         zpos(i) = zpos(i) + EMACH
        else
         zpos(i) = a
        end if
       end do
      end subroutine FOLDTOWSC
C-----------------------------------------------------------------------
      end module midlevel
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
