C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
       module orbital_hf
C   Lithium cluster version; ORBDER and LIDIMER simplified and tested
C   for hybrid wave functions.
C   19.7.2912 WS
       use highlevel
       use midlevel
       use lattice_Lisolid
C                          
       implicit none
C   Introduces constants for HF effective potential orbitals
C   from [Sarsa,Galvez,Buendia].
C   Calculates single electron wavefunctions and its derivatives.
C   Slater parameter SLAP compress wavefunction (1s,2s only)
C   according to exp(zr)-->exp(zr/SLAP).
C
C   MPHI=number of Slater functions
C   NORB=number of orbitals per nucleus
C   NKNORB=total number of orbitals =NK*NORB
C   NELORB(NEMAX)=maps the electron index onto the set of orbitals
       integer,parameter,public          :: MPHI=7,NORB=2,MAXSPL=101
       integer,parameter,public          :: NKNORB=NK*NORB
       integer,dimension(NE),public      :: NELORB
       real(dp),public                   :: SLAP0,SLAPD
       real(dp),public,dimension(NORB)   :: SLAP
       real(dp),public,dimension(NKNORB) :: PSINEW
       real(dp),public,dimension(NK,NORB)             :: WPHI,WHYPHI
C   Internal quantities
       integer,parameter,private                 :: mnphi=2,rint=10
       integer,private                                :: maxi
       integer,private,dimension(MPHI)                :: phinp,fak
       real(dp),private,dimension(MPHI)          :: phizeta,norm
       real(dp),private,dimension(MPHI,NORB)     :: phiaa
       data phinp /2*1,4*2,3/
       data phizeta / 0.72089388_dp, 2.61691643_dp, 0.69257443_dp,
     &  1.37137558_dp, 3.97864549_dp, 13.52900016_dp, 19.30801440_dp/
       data phiaa(1:MPHI,1)/ -0.12220686_dp, 1.11273225_dp,
     &  0.04125378_dp, 0.09306499_dp, -0.10260021_dp, -0.00034191_dp,
     &  0.00021963_dp /
       data phiaa(1:MPHI,2)/ 0.47750469_dp, 0.11140449_dp,
     &  -1.25954273_dp, -0.18475003_dp, -0.02736293_dp, -0.00025064_dp,
     &  0.00057962_dp /
C   Specify routine names
       public :: INITORB,ORBWAV,ORBDER
       public :: LIDIMER
      contains
C-----------------------------------------------------------------------
       subroutine INITORB
C  1. Maps the electron index onto the set of orbitals through the
C     index array i=NELORB(k), i=electron index, i=orbital index via
C     PSINEW(i) depending only on the orbital index i.
C     For Li solid:
C      up-spin electron IE=1,NK;            IEES=IE occupies 1st orbital
C      up-spin electron IE=NK+1,3NK/2;      IEES=IE occupies 2nd orbital
C      down-spin electron IE=3NK/2+1,5NK/2; IEES=IE-NES(1) occ. 1st orb.
C      down-spin electron IE=5NK/2+1,3NK,   IEES=IE-NES(1) occ. 2nd orb.
C     Electrons are distributed upon the nuclei such that the 2nd orbital
C     is occupied for spin IES=1 at the 1st half of the nuclei, i.e.
C     those at integer corrdinates, say at the corner atoms, and for
C     spin IES=2 at the 2nd half of the nuclei, i.e. those at
C     half-integer coordinates and  accordingly at the centers of
C     the bcc nuclear lattice. Thus, each nucleus has 3 electrons with
C     spin-up at the corners and spin-down at the centers for the 2s
C     electron.
C  2. Calculates the factorial, i.e. (2*phinp)! and normalization
C     coefficients.
C  3. Reads orbital compression parameters SLAP(NORB)
C  4. Calculates initial wavefunction matrix PSIMAT(i,k,s) with
C     i=orbital index, k= electron index, s= spin index
        integer,parameter                    :: nk2=NK/2
        integer                              :: j,jj,hies,hi,ii,hk
        integer,dimension(NE)                :: hnelorb
        data hnelorb(1:NK)/NK*1/
        data hnelorb(NK+1:NK+nk2)/nk2*2/
        data hnelorb(NK+nk2+1:2*NK+nk2)/NK*1/
        data hnelorb(2*NK+nk2+1:3*NK)/nk2*2/
C  1. Index array to associate each electron with a wavefunction
        NELORB(1:NE)=hnelorb(1:NE)
C  2. The factorial (2*phinp)! and normalization
        fak=1
        do j=1,MPHI
         do jj=1,2*phinp(j)-1
          fak(j)=fak(j)*(jj+1)
         end do
        end do
        norm=(2.0_dp*phizeta)**(phinp+0.5_dp)/dsqrt(dble(fak))
C  3, Compression parameters
C        write(*,*)'Slater compression parameters, SLAP(1:2)='
C        read (*,*) (SLAP(j),j=1,NORB)
C        write(*,*) (SLAP(j),j=1,NORB)
C  4. The matrix psi_i(k) of the wavefunction is determined.
C     In the columns the index runs over the orbitals i for a
C     fixed electron index k.
C     But one has to discriminate between both spin directions.
C     One begins with spin-up electrons counting them
C     up to the number of electrons NES(1) with this spin.
C     Subsequently follow the spin-down electrons
C     in a new matrix up to NES(2). Take care that the orbital
C     index is correctly associated with the electron index.
C     So we associate for the Li solid the k=1,...,NE=3*NK electrons
C     with the two orbitals at each nucleus
C     in the pair (orbital[nucleus],electron) by
C                           (1[1],1),..........,(1[1],3*NK/2)
C                           .................................
C                           (1[NK],1),.........,(1[NK],3*NK/2)
C                           (2[1],1),.....,(2[NK/2+1],3*NK/2)
C                           .................................
C                           (2[NK/2],1),.........,(2[NK],3*NK/2])
C     for the first matrix, i.e. the spin-up matrix, and by
C                           (1[1],3*NK/2+1),........,(1[1],3*NK)
C                           .................................
C                           (1[NK],3*NK/2+1),.......,(1[NK],3*NK)
C                           (2[NK/2+1],3*NK/2+1),...,(2[NK/2+1],3*NK)
C                           .................................
C                           (2[NK],3*NK/2+1),.......,(2[NK],3*NK)
C     for the second matrix, i.e. the spin-down matrix.
         do j=1,NE
          hies=1
          if (j > NES(1)) hies=2
          hi=(hies-1)*NES(1)
          hk=(hies-1)*NK/2
          call ORBWAV(RKE(4,1:NK,j),WPHI)
          do ii=1,NK
           PSIMAT(ii,j-hi,hies) = WPHI(ii,NELORB(hi+ii))
          end do
           call LIDIMER(WPHI,WHYPHI) !for hybrid states
          do ii=NK+1,3*NK/2
          PSIMAT(ii,j-hi,hies) = WHYPHI(ii-NK+hk,NELORB(hi+ii)) !for hybrid states
C           PSIMAT(ii,j-hi,hies) = WPHI(ii-NK+hk,NELORB(ii))
          end do
         end do
       end subroutine INITORB
C-----------------------------------------------------------------------
       subroutine ORBWAV(rke,wphi)
        real(kind=dp),intent(in),dimension(NK)       :: rke
        real(kind=dp),intent(out),dimension(NK,NORB) :: wphi
C   Calculates the Hartree atomic wavefunctions of Lithium
C   according to [Sarsa,Galvez,Buendia] with respect to each nucleus.
C   Needs more (???%) CPU time than spline interpolation.
        integer             :: j,js
        real(kind=dp)       :: s
        wphi = 0.0_dp
        do j=1,NK
         do js=1,MPHI
           wphi(j,1:NORB)=wphi(j,1:NORB)+phiaa(js,1:NORB)*
     &                         rke(j)**(phinp(js)-1)*norm(js)*
     &                         dexp(-phizeta(js)*rke(j)/SLAP(1:NORB))
         end do
        end do
       end subroutine ORBWAV
C-----------------------------------------------------------------------
       subroutine ORBDER(rke,wphi1,wphi2)
C   Gradient, laplacian, and wave function from HF effective potential
C   atomic wavefunction via [Sarsa,Galvez,Buendia].
        real(dp),intent(in),dimension(4,NK)       :: rke
        real(dp),intent(out),dimension(NK,NORB)   :: wphi2
        real(dp),intent(out),dimension(3,NK,NORB) :: wphi1
C
        integer                        :: i,j,js
        real(dp),dimension(NORB)       :: psid1,psid2
        real(dp),dimension(MPHI,NORB)  :: p1,p2,p3
        real(kind=dp)                   :: s
        do j=1,NK
         s=rke(4,j)
         if (s < EMACH) s=KINEMACH
C  first derivative of wave function
         psid1 = 0.0_dp
         do js=1,MPHI
          p1(js,1:NORB)=phiaa(js,1:NORB)*norm(js)*
     &                      dexp(-phizeta(js)*s/SLAP(1:NORB))
          p2(js,1:NORB)=p1(js,1:NORB)*phizeta(js)/SLAP(1:NORB)*
     &                                              s**(phinp(js)-1)
          p3(js,1:NORB)=p1(js,1:NORB)*(phinp(js)-1)*s**(phinp(js)-2)
          psid1(1:NORB)= psid1(1:NORB) + p3(js,1:NORB) - p2(js,1:NORB)
         end do
C  second derivative of wave function
         psid2 = 0.0_dp
         do js=1,MPHI
          psid2(1:NORB)=psid2(1:NORB) +
     &                  p2(js,1:NORB)*phizeta(js)/SLAP(1:NORB) -
     &           2.0_dp*p3(js,1:NORB)*phizeta(js)/SLAP(1:NORB) +
     &                  p3(js,1:NORB)*(phinp(js)-2)/s
         end do
         do i=1,3
          wphi1(i,j,1:NORB)=rke(i,j)/s*psid1(1:NORB) !gradient of wave function
         end do
         wphi2(j,1:NORB) = psid2(1:NORB) + 2.0_dp/s*psid1(1:NORB) !Laplacian
        end do
       end subroutine ORBDER
C-----------------------------------------------------------------------
      subroutine LIDIMER(w,why)
C   Associates for the 2s states hybrid wavefunctions on both nuclei
C   of the bcc unitcell instead of using only atomic wavefunctions
C   on the 0.5 positions. Has to be called after calling ORBSPLINE and
C   ORBDERSPLINE.
       real(dp),dimension(NK,NORB),intent(in)  :: w
       real(dp),dimension(NK,NORB),intent(out) :: why
C
       integer :: i,k,ik
       integer,dimension(NK/2,NK) :: lcao
C  Below for 2x2x2 only:
C       data lcao(1,1:NK) /NK*1/
C       data lcao(2,1:NK) /4*1,4*-1,8*0/
C       data lcao(3,1:NK) /2*1,2*-1,2*1,2*-1,8*0/
C       data lcao(4,1:NK) /1,-1,1,-1,1,-1,1,-1,8*0/
C       data lcao(5,1:NK) /8*0,4*1,4*-1/
C       data lcao(6,1:NK) /8*0,2*1,2*-1,2*1,2*-1/
C       data lcao(7,1:NK) /8*0,1,-1,1,-1,1,-1,1,-1/
C       data lcao(8,1:NK) /2*1,4*-1,2*1,2*-1,4*1,2*-1/
       why = w
       ik=0
       do i=1,NK/2  !non-vanishing contributions up to SQG=5,real part
        if (GCUTR(i) == 0 .or. SQG(i) > 5) cycle
        ik=ik+1
        why(ik,2) = dot_product (RLCAO(i,1:NK),w(1:NK,2))  
       end do
       do i=1,NK/2   !non-vanishing contributions up to SQG=5,imag.part
        if (GCUTI(i) == 0 .or. SQG(i) > 5) cycle
        ik=ik+1
        why(ik,2) = dot_product (ILCAO(i,1:NK),w(1:NK,2))
       end do
       if (ik /= NK/2) then 
        write (*,*)'number of states not equal NK/2:ik=',ik
        stop
       end if
       do i=1,NK/2
        why(NK/2+i,2) = why(i,2)        
       end do
      end subroutine LIDIMER
C-----------------------------------------------------------------------

      end module orbital_HF
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
