C-----------------------------------------------------------------------
      module jastrow
       use highlevel
       use midlevel
C       use ewald
C  Calculates the change of the Jastrow two-particle potential and
C  updates the derivatives. The Jastrow factor J is written as
C  J=exp(-\sum_k u) with u as in the subroutines.
C  GRJAS(3,NE,IES)=  Gradient of J divided by J for spin IES
C  LAPJAS(NE,IES)= Laplace of J divided by J for spin IES
C  QJ(IES)= J^new/J^old, Jastrow part of acceptance for spin IES,
C  to be squared to yield the rates
C  calculated for actual electron, IE.
C  CJAS= only Jastrow constant for exponential like Jastrow factor
C  BETA1,BETA2= Additional constants for atomic like Jastrow factor,
C  which introduce a short range repulsion (attraction) for
C  positive (negative) values. Set BETA1=0.0 to cancel the latter part.
       implicit none
       public :: INITJAS,JASEXP,JASEXPATOM,JASEXPSOLID
       real(kind=dp),public:: CJAS,BETA1,BETA2,QJ(2),LSMOOTH
       real(kind=dp),dimension(NE,2),public   :: LAPJAS,LAPJASOLD
       real(kind=dp),dimension(3,NE,2),public :: GRJAS,GRJASOLD
      contains
C-----------------------------------------------------------------------
      subroutine INITJAS
       BETA1=0.0_dp
       BETA2=0.0_dp
       LSMOOTH=0.25*SULATCON ! value of Williamson (1996)
      end subroutine INITJAS
C-----------------------------------------------------------------------
       subroutine JASEXP
C  Determines Jastrow exponent and derivatives for a form
C  u=as*F/r*(1-e^(-r/F)), as=0.5 for parallel spin and
C  as=1.0 for antiparallel spins. From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2d,u2do,woo,won
        real(dp),dimension(3) :: u1d,u1do
        jasu=0.0_dp
        jasdif=0.0_dp
        u1d(1:3)=0.0_dp
        u2d=0.0_dp
        call RDIST(RE,RENEW)
        ielek:do k=1,NE
         if (k .eq. IE) then
          cycle ielek
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k < NES(1)))) as =1.0_dp
         woo = DIST(4,IE,k)
         won = DISTNEW(4,IE,k)
         jasdif=jasdif+(as/won*(1.0_dp-dexp(-won/CJAS))-
     &         as/woo*(1.0_dp-dexp(-woo/CJAS)))*CJAS**2
         jasu=jasu+as/won*(1.0_dp-dexp(-won/CJAS))*CJAS**2
         u1d(1:3)=u1d(1:3)-as*DISTNEW(1:3,IE,k)/won**3*(1.0_dp-
     &       dexp(-won/CJAS)*(1.0_dp+won/CJAS))*CJAS**2
         u1do(1:3)=u1do(1:3)-as*DIST(1:3,IE,k)/woo**3*(1.0_dp-
     &       dexp(-woo/CJAS)*(1.0_dp+woo/CJAS))*CJAS**2
         u2d=u2d-as/won*dexp(-won/CJAS)
         u2do=u2do-as/woo*dexp(-woo/CJAS)
        end do ielek
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IE,IES)=-u1d(1:3)
        GRJASOLD(1:3,IE,IES)=-u1do(1:3)
        LAPJAS(IE,IES)=-u2d+(u1d(1)**2+u1d(2)**2+u1d(3)**2)
        LAPJASOLD(IE,IES)=-u2do+(u1do(1)**2+u1do(2)**2+u1do(3)**2)
      end subroutine JASEXP
C-----------------------------------------------------------------------
      subroutine JASEXPATOM
C  Determines Jastrow exponent and derivatives for a form
C  u=F/2/(1+r/F)*(delta_{s,-s'}+1/2*delta_{s,s'})+
C    BETA_1exp(-BETA2*r^2). From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2d,u2do,woo,won
        real(dp),dimension(3) :: u1d,u1do
        jasu=0.0_dp
        jasdif=0.0_dp
        u1d(1:3)=0.0_dp
        u1do(1:3)=0.0_dp
        u2d=0.0_dp
        u2do=0.0_dp
        call RDIST(RE,RENEW)
        ielekat:do k=1,NE
         if (k .eq. IE) then
          cycle ielekat
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k <= NES(1)))) as =1.0_dp
         woo = DIST(4,IE,k)
         won = DISTNEW(4,IE,k)
         jasdif=jasdif+(as/2.0_dp/(1.0_dp+won/CJAS)-
     &       as/2.0_dp/(1.0_dp+woo/CJAS))*CJAS+
     &       BETA1*(dexp(-BETA2*won**2)-dexp(-BETA2*woo**2))
         jasu=jasu+as/2.0_dp/(1.0_dp+won/CJAS)*CJAS+
     &       BETA1*dexp(-BETA2*won**2)
         u1d(1:3)=u1d(1:3)-as*DISTNEW(1:3,IE,k)/won/
     &            (1.0_dp+won/CJAS)**2/2.0_dp-
     &       2.0_dp*DISTNEW(1:3,IE,k)*BETA1*BETA2*dexp(-BETA2*won**2)
         u1do(1:3)=u1do(1:3)-as*DIST(1:3,IE,k)/woo/
     &            (1.0_dp+woo/CJAS)**2/2.0_dp-
     &       2.0_dp*DIST(1:3,IE,k)*BETA1*BETA2*dexp(-BETA2*woo**2)
         u2d=u2d-as/won/(1.0_dp+won/CJAS)**3+
     &    BETA1*BETA2*dexp(-BETA2*won**2)*(4.0_dp*BETA2*won**2-6.0_dp)
         u2do=u2do-as/woo/(1.0_dp+woo/CJAS)**3+
     &    BETA1*BETA2*dexp(-BETA2*woo**2)*(4.0_dp*BETA2*woo**2-6.0_dp)
        end do ielekat
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IEES,IES)=-u1d(1:3)
        GRJASOLD(1:3,IEES,IES)=-u1do(1:3)
        LAPJAS(IEES,IES)=-u2d+(u1d(1)**2+u1d(2)**2+u1d(3)**2)
        LAPJASOLD(IEES,IES)=-u2do+(u1do(1)**2+u1do(2)**2+u1do(3)**2)
       end subroutine JASEXPATOM
C-----------------------------------------------------------------------
       subroutine JASEXPSOLID
C  Determines Jastrow exponent and derivatives for a form
C  u=as*CJAS**2/r*(1-e^(-r/CJAS)), as=0.5 for parallel spin and
C  as=1.0 for antiparallel spins. From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained. Here, the harmonic potential is not included to
C  get a continuous derivative (Gradient). The exponential term
C  of the Jastrow factor is neglected in the Ewald sum, because
C  of rapid decay for distances beyond the Wigner-Seitz cell.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2dn,u2do,woo,won,jasewo,jasewn
        real(dp) :: hao,han,hbo,hbn,smootho,smoothn,hl1o,hl1n,hl2o,hl2n
        real(dp),dimension(3) :: u1dn,u1do,jasew1o,jasew1n,zeo,zen
        jasu=0.0_dp
        jasdif=0.0_dp
        u1dn(1:3)=0.0_dp
        u2dn=0.0_dp
        u1do(1:3)=0.0_dp
        u2do=0.0_dp
        ieleksol:do k=1,NE
         if (k .eq. IE) then
          cycle ieleksol
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k <= NES(1)))) as =1.0_dp
C  Minimum Coulomb convention: fold difference into WSC
         zeo(1:3) = DIST(1:3,IE,k)/SULATCON
         zen(1:3) = DISTNEW(1:3,IE,k)/SULATCON
         call FOLDTOWSC(zeo(1:3))
         call FOLDTOWSC(zen(1:3))
         zeo(1:3) = zeo*SULATCON
         zen(1:3) = zen*SULATCON
         woo = sqrt (sum (zeo(1:3)**2))
         won = sqrt (sum (zen(1:3)**2))
C  Smooth the WSC cut-off by exp(-r**2/LSMOOTH**2)
         smootho = dexp(-woo**2/LSMOOTH**2)
         smoothn = dexp(-won**2/LSMOOTH**2)
         hl1o = 1._dp - 4._dp*woo*CJAS/LSMOOTH**2
         hl1n = 1._dp - 4._dp*won*CJAS/LSMOOTH**2
         hl2o = (2._dp - 4._dp*woo**2/LSMOOTH**2)/LSMOOTH**2
         hl2n = (2._dp - 4._dp*won**2/LSMOOTH**2)/LSMOOTH**2
C         woo = DIST(4,IE,k)   !obsolete
C         won = DISTNEW(4,IE,k)  !obsolete
C  Include the Ewald energy for the probability
C         jasewo=VEW(SULATTY,DIST(1:3,IE,k)/SULATCON)/SULATCON
C         jasewn=VEW(SULATTY,DISTNEW(1:3,IE,k)/SULATCON)/SULATCON
         jasewo=0.0_dp
         jasewn=0.0_dp
         han=(1.0_dp-dexp(-won/CJAS))/won*smoothn
         hao=(1.0_dp-dexp(-woo/CJAS))/woo*smootho
         jasdif=jasdif + as*CJAS**2*((han+jasewn)-(hao+jasewo))
         jasu=jasu + as*CJAS**2*(han+jasewn)
C  The kinetic energy needs Gradient with Ewald
C         call VEWGR(SULATTY,DIST(1:3,IE,k)/SULATCON,jasew1o)
C         call VEWGR(SULATTY,DISTNEW(1:3,IE,k)/SULATCON,jasew1n)
C         jasew1n=jasew1n/SULATCON**2
C         jasew1o=jasew1o/SULATCON**2
         jasew1n=0.0_dp
         jasew1o=0.0_dp
         hbn=smoothn/won**2*(1.0_dp-dexp(-won/CJAS)*(1.0_dp+won/CJAS))
         hbo=smootho/woo**2*(1.0_dp-dexp(-woo/CJAS)*(1.0_dp+woo/CJAS))
         u1dn(1:3)=u1dn(1:3)-as*CJAS**2*zen(1:3)/won*
     &      (hbn+2._dp*won/LSMOOTH**2*han) - as*CJAS**2*jasew1n(1:3)
         u1do(1:3)=u1do(1:3)-as*CJAS**2*zeo(1:3)/woo*
     &      (hbo+2._dp*woo/LSMOOTH**2*hao) - as*CJAS**2*jasew1o(1:3)
C  The Laplacian; its Ewald contribution vanishes because of the
C  absence of sources in the primitive superlattice cell
         u2dn=u2dn-as/won*dexp(-won/CJAS)*smoothn*hl1n +
     &      as*CJAS**2*smoothn*han*hl2n
         u2do=u2do-as/woo*dexp(-woo/CJAS)*smootho*hl2o +
     &      as*CJAS**2*smootho*hao*hl2o
        end do ieleksol
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IEES,IES)=-u1dn(1:3)
        GRJASOLD(1:3,IEES,IES)=-u1do(1:3)
        LAPJAS(IEES,IES)=-u2dn+sum (u1dn(1:3)**2)
        LAPJASOLD(IEES,IES)=-u2do+sum (u1do(1:3)**2)
      end subroutine JASEXPSOLID
C-----------------------------------------------------------------------
      end module jastrow
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
